//
//  ViewController.swift
//  PragmaticTweets
//
//  Created by Chris Adamson on 10/19/14.
//  Copyright (c) 2014 Pragmatic Programmers, LLC. All rights reserved.
//

import UIKit
import Social
import Accounts

public class ViewController: UITableViewController {

  var parsedTweets : [ParsedTweet] = []
  
  override public func viewDidLoad() {
    super.viewDidLoad()
    reloadTweets()
    var refresher = UIRefreshControl()
    refresher.addTarget(self,
      action: "handleRefresh:",
      forControlEvents: UIControlEvents.ValueChanged)
    refreshControl = refresher
  }


  override public func didReceiveMemoryWarning() {
    super.didReceiveMemoryWarning()
    // Dispose of any resources that can be recreated.
  }
  

 @IBAction func handleTweetButtonTapped(sender: UIButton) {
    if SLComposeViewController.isAvailableForServiceType(SLServiceTypeTwitter) {
      let tweetVC = SLComposeViewController (forServiceType: SLServiceTypeTwitter)
      let message = NSLocalizedString(
        "I just finished the first project in iOS 8 SDK Development. #pragsios8",
        comment:"")
        tweetVC.setInitialText(message)
      presentViewController(tweetVC, animated: true, completion: nil)
    } else {
      println ("Can't send tweet")
    }
  }
  
  func reloadTweets() {
    let accountStore = ACAccountStore()
    let twitterAccountType = accountStore.accountTypeWithAccountTypeIdentifier(
      ACAccountTypeIdentifierTwitter)
    accountStore.requestAccessToAccountsWithType(twitterAccountType,
      options: nil,
      completion: {
        (granted: Bool, error: NSError!) -> Void in
        if (!granted) {
          println ("account access not granted")
        } else {
          let twitterAccounts = accountStore.accountsWithAccountType(twitterAccountType)
          if twitterAccounts.count == 0 {
            println ("no twitter accounts configured")
            return
          } else {
            let twitterParams = [
              "count" : "100"
            ]
            let twitterAPIURL = NSURL(string:
              "https://api.twitter.com/1.1/statuses/home_timeline.json")
            let request = SLRequest(forServiceType: SLServiceTypeTwitter,
              requestMethod:SLRequestMethod.GET,
              URL:twitterAPIURL,
              parameters:twitterParams)
            request.account = twitterAccounts.first as! ACAccount
            request.performRequestWithHandler ( {
              (data: NSData!, urlResponse: NSHTTPURLResponse!, error: NSError!) -> Void in
            self.handleTwitterData(data, urlResponse: urlResponse, error: error)
            })
        }
      }
    })
  }
  
  func handleTwitterData (data: NSData!,
      urlResponse: NSHTTPURLResponse!,
      error: NSError!) {
    if let dataValue = data {
        var parseError : NSError? = nil
      let jsonObject : AnyObject? =
        NSJSONSerialization.JSONObjectWithData(dataValue,
        options: NSJSONReadingOptions(0),
        error: &parseError)
      if parseError != nil {
        return
      }
      if let jsonArray = jsonObject as? [ [String:AnyObject] ] {
        self.parsedTweets.removeAll(keepCapacity: true)
        for tweetDict in jsonArray {
          let parsedTweet = ParsedTweet()
          parsedTweet.tweetText = tweetDict["text"] as? String
          parsedTweet.createdAt = tweetDict["created_at"] as? String
          let userDict = tweetDict["user"] as! NSDictionary
          parsedTweet.userName = userDict["name"] as? String
          parsedTweet.userAvatarURL = NSURL (string:
            userDict ["profile_image_url"] as! String)
          self.parsedTweets.append(parsedTweet)
        }
        dispatch_async(dispatch_get_main_queue(),
          { 
            self.tableView.reloadData()
          })
        }
      } else {
        println ("handleTwitterData received no data")
      }
    }

  override public func numberOfSectionsInTableView(tableView: UITableView) -> Int {
    return 1
  }
  
  override public func tableView(_tableView: UITableView,
    numberOfRowsInSection section: Int) -> Int {
      return parsedTweets.count
  }
 
  override public func tableView (_tableView: UITableView,
    cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
      let cell =
      tableView.dequeueReusableCellWithIdentifier("CustomTweetCell")
        as! ParsedTweetCell
      let parsedTweet = parsedTweets[indexPath.row]
      cell.userNameLabel.text = parsedTweet.userName
      cell.tweetTextLabel.text = parsedTweet.tweetText
      cell.createdAtLabel.text = parsedTweet.createdAt
      if parsedTweet.userAvatarURL != nil {
        /* fast, wrong version
        dispatch_async(dispatch_get_global_queue(  
          QOS_CLASS_DEFAULT, 0), 
          { 
            if let imageData = NSData (contentsOfURL: 
              parsedTweet.userAvatarURL!) { 
            let avatarImage = UIImage(data: imageData) 
            dispatch_async(dispatch_get_main_queue(),  
            {
              cell.avatarImageView.image = avatarImage
            }) 
          }
        }) 
        */

        /* fast, right version
        */
        cell.avatarImageView.image = nil 
        dispatch_async(dispatch_get_global_queue(
          QOS_CLASS_DEFAULT, 0),
            {
              if let imageData = NSData (contentsOfURL:
                parsedTweet.userAvatarURL!) {
              let avatarImage = UIImage(data: imageData)
              dispatch_async(dispatch_get_main_queue(), 
                {
                  if cell.userNameLabel.text == parsedTweet.userName {
                    cell.avatarImageView.image = avatarImage
                  } else {
                    println ("oops, wrong cell, never mind")
                  }
                }) 
              }
          })
      }
      return cell
  }

  @IBAction func handleRefresh (sender : AnyObject?) {
    reloadTweets()
    refreshControl!.endRefreshing()
  }
  
}

